// EPOS Cortex-M3 RS-485 Mediator Declarations

#ifndef __cortex_m_rs485_h__
#define __cortex_m_rs485_h__

#include "uart.h"
#include "gpio.h"

__BEGIN_SYS

class RS485_Transceiver: protected Machine_Model
{
private:
    static const char RE_PORT = 'C';
    static const unsigned int RE_PIN  = 5;
    static const char DE_PORT = 'C';
    static const unsigned int DE_PIN  = 6;

public:
    RS485_Transceiver(): _re(GPIO(RE_PORT, RE_PIN, GPIO::OUT)), _de(GPIO(DE_PORT, DE_PIN, GPIO::OUT)) {}

    void out() { _re.set(true); _de.set(true); }
    void in() { _re.set(false); _de.set(false); }

private:
    GPIO _re;
    GPIO _de;
};

class RS485: private UART, private RS485_Transceiver
{
private:
    typedef RS485_Transceiver Transceiver;
    typedef UART Engine;

    static const unsigned int UNIT = Traits<UART>::DEF_UNIT;
    static const unsigned int BAUD_RATE = Traits<UART>::DEF_BAUD_RATE;
    static const unsigned int DATA_BITS = Traits<UART>::DEF_DATA_BITS;
    static const unsigned int PARITY = Traits<UART>::DEF_PARITY;
    static const unsigned int STOP_BITS = Traits<UART>::DEF_STOP_BITS;

public:
    RS485(unsigned int unit = UNIT, unsigned int baud_rate = BAUD_RATE, unsigned int data_bits = DATA_BITS, unsigned int parity = PARITY, unsigned int stop_bits = STOP_BITS)
    : Engine(unit, baud_rate, data_bits, parity, stop_bits) {}

    void config(unsigned int baud_rate, unsigned int data_bits, unsigned int parity, unsigned int stop_bits) {
        Engine::config(baud_rate, data_bits, parity, stop_bits);
    }
    void config(unsigned int * baud_rate, unsigned int * data_bits, unsigned int * parity, unsigned int * stop_bits) {
        Engine::config(*baud_rate, *data_bits, *parity, *stop_bits);
    }

    int read(void * data, unsigned int size) {
        char * d = reinterpret_cast<char *>(data);
        unsigned int s = 0;

        for(; s < size; s++) {
            if(ready_to_get())
                d[s] = get();
            else
                break;
        }
        return s;
    }
    int write(const void * data, unsigned int size) {
        const char * d = reinterpret_cast<const char *>(data);
        unsigned int s = 0;

        Transceiver::out();
        for(; s < size; s++)
            put(d[s]);
        Engine::flush();
        Transceiver::in();

        return s;
    }

    using Engine::flush;
    using Engine::ready_to_get;
    using Engine::ready_to_put;

    using Engine::int_enable;
    using Engine::int_disable;

    using Engine::reset;
    using Engine::loopback;

    using Engine::power;
};

__END_SYS

#endif
